/////////////////////////////////////////////////////////////
// cacheb.h: Resource cache base class definitions.
// This is the data independent code.
// Copyright(c) 1993 Azarona Software. All rights reserved.
/////////////////////////////////////////////////////////////
#ifndef H_CACHEB
#define H_CACHEB
#include "fmgr.h"

class Bucketb; class Coptrb; // Forward decl's

class Cacheb {
protected:
  long hits, misses; // For performance statistics
  long fast_binds;
  int nbuckets;      // Size of cache
  FmgrPtr fptr;      // File cache is connected to
  Bucketb *head;     // Most recently reserved bucket
  Bucketb *AcquireBkt();
  Bucketb *FindBkt(long addr);
  void MoveToFront(Bucketb *b);
  void Setup(Bucketb *b, int n, unsigned bkt_size);
  Cacheb(Bucketb *b, int n, unsigned bkt_size);
  virtual ~Cacheb();
public:
  friend class Coptrb;
  void Connect(FmgrPtr &fp);
  void Disconnect();
  void Flush(int empty_bkts = 0);
  void Clear();
  virtual Bucketb *ReserveBkt(long addr, int ensure_loaded=1);
  void FastBind();
  friend void Report(Cacheb &c, int full_report=0);
};

inline Cacheb::~Cacheb()
{
  // Derived class should do all the work
}

inline void Cacheb::FastBind()
{
  fast_binds++;
}

#endif
