///////////////////////////////////////////////////////////////
// fmgrptr.h: File manager smart pointer class.
// *** THIS FILE BASED ON SMARTPTR.H TEMPLATE FILE ***
// Copyright(c) 1993 Azarona Software. All rights reserved.
/////////////////////////////////////////////////////////////
#ifndef H_FMGRPTR
#define H_FMGRPTR

class Fmgr; // Forward decl

class FmgrPtr {
// Fmgr is assumed to be a structure having
// member functions IncRef(), DecRef(), and
// UnReferenced().
protected:
  Fmgr *objptr;
  void Bind(const FmgrPtr &p);
  void Unbind();
  void NewBinding(const FmgrPtr &p);
public:
  FmgrPtr(Fmgr *p=0);
  FmgrPtr(const FmgrPtr &p);
  void operator=(const FmgrPtr &p);
  void operator=(int dmy);
  ~FmgrPtr();
  Fmgr &operator*() const;
  Fmgr *operator->() const;
  int operator!() const;
  operator int() const;
  friend int operator==(const FmgrPtr &a, const FmgrPtr &b);
  friend int operator!=(const FmgrPtr &a, const FmgrPtr &b);
};

inline FmgrPtr::FmgrPtr(Fmgr *p)
// Constructor that can be used as a default
{
  objptr = p;
}

inline FmgrPtr::FmgrPtr(const FmgrPtr &p)
// Copy constructor using share semantics
{
  Bind(p);
}

inline FmgrPtr::~FmgrPtr()
// Destroys a smart pointer by unbinding what
// it's bound to.
{
  Unbind();
}

inline void FmgrPtr::operator=(const FmgrPtr &p)
// Assignment operator. Uses share semantics.
{
  NewBinding(p);
}

inline void FmgrPtr::operator=(int)
// The dmy parameter (usually 0) flags us to set the pointer
// to "null."
{
  Unbind();
  objptr = 0;
}

inline Fmgr &FmgrPtr::operator*() const
// Does the equivalent of a target object dereference.
{
  return *objptr;
}

inline Fmgr *FmgrPtr::operator->() const
// Target object structure pointer dereference.
{
  return objptr;
}

inline int operator==(const FmgrPtr &a, const FmgrPtr &b)
{
  return a.objptr == b.objptr;
}

inline int operator!=(const FmgrPtr &a, const FmgrPtr &b)
{
  return a.objptr != b.objptr;
}

int inline FmgrPtr::operator!() const
// Returns 1 if pointer references null_obj, else 0
{
  return objptr == 0;
}

inline FmgrPtr::operator int() const
// Returns 1 if pointer doesn't reference null_obj, else 0
{
  return objptr != 0;
}

#endif
